#include <stdlib.h>
#include <vector>
#include <iostream>

using namespace std;

// Défintion du type graphe :
/* Un graphe est représenté par sa matrice d'adjacence 
   c'est à dire un un tableau de tableaux d'entiers */

typedef vector<vector<int> > graphe;


// Créé et renvoie le graphe à n sommets et sans arête
graphe graphe_vide(int n){
  return graphe (n, vector<int>(n, 0));
}

// Fonctions qui vérifie si une matrice est symétrique
int test_symetrique(graphe &g){
  for (int i = 0; i < g.size (); i++){
    if ( g[i].size() != g.size () ) return 0;
    for (int j = 0; j < i; j++){
      if ( g[i][j] != g[j][i] ) return 0;
    }
  }
  return 1;
}

// Fonction qui teste si la diagonale de la matrice est nulle
int test_diagonale(graphe &g){
  for (int i = 0; i < g.size (); i++)
    if (g[i][i] != 0) return 0;
  return 1;
}

// Fonction qui teste si l'entrée est bien une matrice d'adjacence correcte
bool est_graphe(graphe &g)
{
    return (test_symetrique(g) && test_diagonale(g));
}

// Affichage d'une matrice d'un graphe
void affiche(graphe &g, string nom){
  int n = g.size ();
  // En-tête
  cout << "Affichage du graphe " << nom;
  cout << " à " << g.size() << " sommets :" << endl << endl;

  // Écrit un avertissement si la matrice n'est pas celle d'un graphe
  if ( not test_symetrique(g) ){
    cout << "Attention !!! La matrice n'est pas symétrique." << endl;
    cout.flush ();
  }
  if ( not test_diagonale(g) ){
    cout << "Attention !!! La diagonale n'est pas nulle partout." << endl;
    cout.flush ();
  }

  // Affichage de la matrice
  cout << "Matrice d'adjacence :" << endl;
  for (int i = 0; i < n; i++){
    for (int j = 0; j < n; j++){
      cout << g[i][j] << " ";
    }
    cout << endl;
  }

  // Affichage de la liste des arêtes
  cout << endl << "arêtes : ";
  for (int i = 0; i < n; i++){
    for (int j = i+1; j < n; j++){
      if (g[i][j])
	cout << "(" << i << "," << j << ") ";
    }
  }
  cout << endl << endl;
}
